﻿Imports System.Security.Cryptography
Imports System.Text
Imports System.IO
Imports Microsoft.Win32
Imports SciLors_Logger

Module Utilities
    Public myConfig As New Settings

    Public Class Settings
        Private Const SettingsFileName As String = "config.xml"


        Public Enum ExistingFileHandling
            Skip = 0
            Overwrite = 1
            Rename = 2
        End Enum

        Public Structure Config
            Dim DownloadDir As String
            Dim NamePattern As String
            Dim UncheckAfterAdd As Boolean
            Dim AutoCheckUpdates As Boolean
            Dim ExistingFileHandling As ExistingFileHandling
            Dim Proxy As String
            Dim ProxyUser As String
            Dim ProxyPass As String

        End Structure

        Public Settings As Config

        Sub New()
            Call Load()
        End Sub

        Private Function GetKey() As String
            Try
                Dim FirstPart As String = Registry.LocalMachine.OpenSubKey("SOFTWARE").OpenSubKey("Microsoft").OpenSubKey("Windows NT").OpenSubKey("CurrentVersion").GetValue("BuildGUID", "!%&/()=?%&/()jkfaf²³²³")
                Dim SecondPart As String = Registry.LocalMachine.OpenSubKey("SOFTWARE").OpenSubKey("Microsoft").OpenSubKey("Windows NT").OpenSubKey("CurrentVersion").GetValue("ProductId", "³³³³³2412³²}\$%/()=")
                Return SecondPart + FirstPart
            Catch ex As Exception
                MessageBox.Show(ex.Message)
                Return "$%/()=56856879080897%&)(/=kjblfono8B(/O)(P)B="
            End Try
            Return ("Hello")
        End Function

        Private Function Encode(ByVal text As String) As String
            If text.Length = 0 Then
                Return ""
            End If

            Dim rm As New RijndaelManaged

            Dim md5 As New MD5CryptoServiceProvider
            Dim key() As Byte = md5.ComputeHash(Encoding.UTF8.GetBytes(GetKey()))

            md5.Clear()
            rm.Key = key
            rm.GenerateIV()

            Dim ms As New MemoryStream
            ms.Write(rm.IV, 0, rm.IV.Length)

            Dim cs As New CryptoStream(ms, rm.CreateEncryptor, CryptoStreamMode.Write)
            Dim data() As Byte = System.Text.Encoding.UTF8.GetBytes(text)

            cs.Write(data, 0, data.Length)
            cs.FlushFinalBlock()

            Return Convert.ToBase64String(ms.ToArray())
        End Function

        Private Function Decode(ByVal text As String) As String
            If text.Length = 0 Then
                Return ""
            End If

            Dim rm As New RijndaelManaged
            Dim IvLength As Integer = 16
            Dim md5 As New MD5CryptoServiceProvider
            Dim key() As Byte = md5.ComputeHash(Encoding.UTF8.GetBytes(GetKey))

            md5.Clear()
            Dim ms As New MemoryStream(Convert.FromBase64String(text))
            Dim iv(IvLength - 1) As Byte

            ms.Read(iv, 0, IvLength)
            rm.IV = iv
            rm.Key = key

            Dim cs As New CryptoStream(ms, rm.CreateDecryptor, CryptoStreamMode.Read)

            Dim data(ms.Length - IvLength) As Byte
            Dim i As Integer = cs.Read(data, 0, data.Length)

            text = System.Text.Encoding.UTF8.GetString(data, 0, i)
            cs.Close()
            rm.Clear()

            Return text
        End Function

        Public Sub SetDefaults()
            Settings.DownloadDir = IO.Path.Combine(GetAppPath, "Downloads")
            Settings.NamePattern = "<artist> - <song> - <album>"
            Settings.UncheckAfterAdd = True
            Settings.AutoCheckUpdates = True
            Settings.ExistingFileHandling = ExistingFileHandling.Skip
            Settings.Proxy = ""
            Settings.ProxyUser = ""
            Settings.ProxyPass = ""
        End Sub

        Public Sub Load()
            Call SetDefaults()
            Try
                Dim xd As XDocument = XDocument.Load(IO.Path.Combine(GetAppPath, SettingsFileName))
                Settings.DownloadDir = xd.Root.Element("DownloadDir").Value
                Settings.NamePattern = xd.Root.Element("NamePattern").Value
                Settings.UncheckAfterAdd = xd.Root.Element("UncheckAfterAdd").Value
                Settings.AutoCheckUpdates = xd.Root.Element("AutoCheckUpdates").Value
                Settings.ExistingFileHandling = [Enum].Parse(GetType(ExistingFileHandling), xd.Root.Element("ExistingFileHandling").Value)
                Settings.Proxy = xd.Root.Element("Proxy").Value
                Settings.ProxyUser = xd.Root.Element("ProxyUser").Value
                Settings.ProxyPass = Decode(xd.Root.Element("ProxyPass").Value)
            Catch
                Call Save()
            End Try
        End Sub

        Public Sub Save()
            Dim xd As New XDocument(New XElement("SciLorsGrooveDownloaderSettings", _
                                    New XElement("DownloadDir", Settings.DownloadDir), _
                                    New XElement("NamePattern", Settings.NamePattern), _
                                    New XElement("UncheckAfterAdd", Settings.UncheckAfterAdd), _
                                    New XElement("AutoCheckUpdates", Settings.AutoCheckUpdates), _
                                    New XElement("ExistingFileHandling", Settings.ExistingFileHandling.ToString()), _
                                    New XElement("Proxy", Settings.Proxy), _
                                    New XElement("ProxyUser", Settings.ProxyUser), _
                                    New XElement("ProxyPass", Encode(Settings.ProxyPass))
                                    ))
            Try
                xd.Save(IO.Path.Combine(GetAppPath, SettingsFileName))
            Catch ex As UnauthorizedAccessException
                AdministrationRightsNeeded()
            End Try
        End Sub
    End Class


    Public Function GetAppPath() As String
        GetAppPath = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase)
        If GetAppPath.StartsWith("file:\") Then
            GetAppPath = GetAppPath.Substring(6)
        End If
    End Function

    Public log As New Logger

    Public Sub EnableDebugLog()
        log = New Logger(Path.Combine(GetAppPath(), "SciLorsGrooveDownloader.log"), Logger.LogLevel.ALL)
    End Sub

    Public Function SongRenamePattern(ByVal Pattern As String, ByVal Song As SciLorsGroovesharkAPI.Groove.Functions.SearchArtist.SearchArtistResult, Optional ByVal AutoRenameString As String = "") As String
        On Error Resume Next
        Pattern = Pattern.Replace("<artist>", StripIllegalFileName(Song.ArtistName))
        Pattern = Pattern.Replace("<song>", StripIllegalFileName(Song.SongName))
        Pattern = Pattern.Replace("<album>", StripIllegalFileName(Song.AlbumName))
        Pattern = Pattern.Replace("<year>", StripIllegalFileName(Song.Year))
        Pattern = Pattern.Replace("<tracknum>", StripIllegalFileName(Song.TrackNum))


        Return StripIllegalPath(Pattern) & AutoRenameString & ".mp3"
    End Function


    Public Sub AdministrationRightsNeeded()
        If MsgBox("You may have to run this application under administration rights" & vbNewLine & "Start as administrator?", MsgBoxStyle.YesNo) = MsgBoxResult.Yes Then
            Dim info As New ProcessStartInfo
            info.FileName = System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase
            info.UseShellExecute = True
            info.Verb = "runas"
            Try
                Process.Start(info)
            Catch
                MsgBox("No administration rights, closing...", MsgBoxStyle.Critical)
            End Try
            Application.Current.Shutdown()
        End If
    End Sub

    Public Function StripIllegalPath(ByVal myString As String) As String
        If myString IsNot Nothing Then
            For Each Character As Char In IO.Path.GetInvalidPathChars
                myString = myString.Replace(Character, "")
            Next
        End If
        Return myString
    End Function

    Public Function StripIllegalFileName(ByVal myString As String) As String
        If myString IsNot Nothing Then
            For Each Character As Char In IO.Path.GetInvalidFileNameChars
                myString = myString.Replace(Character, "")
            Next
        End If
        Return myString
    End Function
End Module
